"""
w2lapp.groupadm.py: add/delete user entry to/from group entries

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: groupadm.py,v 1.64 2009/08/02 12:31:05 michael Exp $
"""

import ldap,ldap.cidict,ldaputil.base, \
       pyweblib.forms,w2lapp.core,w2lapp.gui

from ldaputil.base import SplitRDN

ACTION2MODTYPE = {'add':ldap.MOD_ADD,'remove':ldap.MOD_DELETE}

REQUESTED_GROUP_ATTRS = ['objectClass','cn','description']


def GroupSelectFieldHTML(ls,form,all_groups_dict,field_name,field_title,group_search_root,dn_list):
  option_list = []
  for dn in dn_list:
#    dn = unicode(dn,ls.charset)
    option_text = unicode(
      all_groups_dict[dn].get(
        'cn',[dn[:-len(group_search_root) or len(dn)].encode(ls.charset)]
      )[0],
      ls.charset
    )
    option_title = unicode(
      all_groups_dict[dn].get(
        'description',[dn[:-len(group_search_root)].encode(ls.charset)]
      )[0],
      ls.charset
    )
    option_list.append(
      (u'<option value="%s" title="%s">%s</option>' % (
        dn,option_title,option_text
      )).encode(form.accept_charset)
    )
  return '<select size="15" multiple name="%s" title="%s">\n%s\n</select>\n' % (
    field_name,
    field_title,
    '\n'.join(option_list)
  )


def w2l_GroupAdm(sid,outf,command,form,ls,dn):

  groupadm_defs = ldap.cidict.cidict(w2lapp.cnf.GetParam(ls,'groupadm_defs',{}))
  if not groupadm_defs:
    raise w2lapp.core.ErrorExit('Group admin options empty or not set.')
  groupadm_defs_keys = groupadm_defs.keys()

  all_membership_attrs = [
    user_entry_attrtype
    for group_member_attrtype,user_entry_attrtype in groupadm_defs.values()
    if not user_entry_attrtype is None
  ]

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))

  result_dnlist = ls.readEntry(dn,all_membership_attrs)
  if not result_dnlist:
    raise w2lapp.core.ErrorExit('No search result when reading entry.')

  user_entry = ldap.schema.Entry(sub_schema,dn,result_dnlist[0][1])

  # Extract form parameters
  group_search_root = form.getInputValue('group_search_root',[ls.getSearchRoot(dn)])[0]
  groupadm_view = int(form.getInputValue('groupadm_view',['1'])[0])
  
  filter_components = []
  for oc in groupadm_defs.keys():
    group_member_attrtype,user_entry_attrtype = groupadm_defs[oc]
    if user_entry_attrtype is None:
      user_entry_attrvalue = dn.encode(ls.charset)
    else:
      try:
        user_entry_attrvalue = user_entry[user_entry_attrtype][0]
      except KeyError:
        continue
    filter_components.append(
      (
        oc.strip(),
        group_member_attrtype.strip(),
        ldap.filter.escape_filter_chars(user_entry_attrvalue)
      )
    )

  #################################################################
  # Search all the group entries
  #################################################################

  all_group_filterstr = '(|%s)' % (''.join(
    [
      '(objectClass=%s)' % (oc)
      for oc,attr_type,attr_value in filter_components
    ]
  ))
    
  all_groups_dict = {}

  try:
    for group_dn,group_entry in ls.l.search_ext_s(
      group_search_root.encode(ls.charset),
      ldap.SCOPE_SUBTREE,
      all_group_filterstr,
      attrlist=REQUESTED_GROUP_ATTRS,attrsonly=0,timeout=ls.timeout,nocache=0
    ):
      if group_dn!=None:
        all_groups_dict[unicode(group_dn,ls.charset)] = ldap.cidict.cidict(group_entry)
  except ldap.NO_SUCH_OBJECT:
    pass

  all_group_entries = all_groups_dict.keys()
  all_group_entries.sort()

  #################################################################
  # Apply changes to group membership
  #################################################################

  if 'groupadm_add' in form.inputFieldNames or \
     'groupadm_remove' in form.inputFieldNames:

    for action in ['add','remove']:
      for action_group_dn in form.getInputValue('groupadm_%s'%action,[]):
        group_dn = action_group_dn
        assert all_groups_dict.has_key(group_dn)
        if not all_groups_dict.has_key(group_dn):
          # The group entry could have been removed in the mean time
          # => Ignore that condition
          continue
        modlist = []
        for oc in groupadm_defs_keys:
          if oc.lower() in [ v.lower() for v in all_groups_dict[group_dn]['objectClass'] ]:
            group_member_attrtype,user_entry_attrtype = groupadm_defs[oc]
            if user_entry_attrtype is None:
              member_value = dn.encode(ls.charset)
            else:
              if not user_entry.has_key(user_entry_attrtype):
                raise w2lapp.core.ErrorExit("""
                  Object class %s requires entry to have member attribute %s.""" % (
                    oc,user_entry_attrtype
                  )
                )                
              member_value = user_entry[user_entry_attrtype][0]
            modlist.append((ACTION2MODTYPE[action],group_member_attrtype,member_value))
        ls.modifyEntry(group_dn,modlist)

  #################################################################
  # Search for groups the entry is member of
  #################################################################

  remove_group_filterstr = '(|%s)' % (''.join(
    [
      '(&(objectClass=%s)(%s=%s))' % (oc,attr_type,attr_value)
      for oc,attr_type,attr_value in filter_components
    ]
  ))

  remove_groups_dict = {}
  
  try:
    for group_dn,group_entry in ls.l.search_ext_s(
      group_search_root.encode(ls.charset),
      ldap.SCOPE_SUBTREE,
      remove_group_filterstr,
      attrlist=[],attrsonly=0,timeout=ls.timeout,nocache=1
    ):
      if group_dn!=None:
        remove_groups_dict[unicode(group_dn,ls.charset)] = None
  except ldap.NO_SUCH_OBJECT:
    pass
  remove_group_dns = remove_groups_dict.keys()
  remove_group_dns.sort()

  remove_groups = [ group_dn for group_dn in remove_group_dns ]

  #########################################################
  # Sort out groups the entry is not(!) a member of
  #########################################################

  add_groups = [
    group_dn
    for group_dn in all_group_entries
    if not remove_groups_dict.has_key(group_dn)
  ]

  #########################################################
  # HTML output
  #########################################################

  w2lapp.gui.TopSection(
    sid,outf,form,ls,dn,
    'Group selection table',
    w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=[]
  )

  group_search_root_field = w2lapp.gui.SearchRootField(
    form,ls,dn,name='group_search_root'
  )
  group_search_root_field.charset = form.accept_charset
  group_search_root_field.setDefault(group_search_root)

  outf.write('<div id="Message" class="Main">')

  if all_groups_dict:
    outf.write("""
      <form action="%s/groupadm/%s"
            method="POST"
            enctype="application/x-www-form-urlencoded"
            accept-charset="%s"
      >
        <input type="hidden" name="dn" value="%s">
        <input type="hidden" name="group_search_root" value="%s">
        <input type="submit" value="Change Group Membership">
        <table summary="Group select fields">
          <tr>
            <td width="50%%">Add to...</td>
            <td width="50%%">Remove from...</td>
          </tr>
          <tr>
            <td width="50%%">%s</td>
            <td width="50%%">%s</td>
          </tr>
        </table>
      </form>
    """ % (
      # form for changing group membership
      form.script_name,sid,form.accept_charset,
      w2lapp.core.utf2display(form.accept_charset,dn),
      w2lapp.core.utf2display(form.accept_charset,group_search_root),
      GroupSelectFieldHTML(ls,form,all_groups_dict,'groupadm_add','Groups to add to',group_search_root,add_groups),
      GroupSelectFieldHTML(ls,form,all_groups_dict,'groupadm_remove','Groups to remove from',group_search_root,remove_groups),
    ))
  else:
    outf.write('No group entries found below %s. You might want to select a different group search root' % (
      w2lapp.core.utf2display(form.accept_charset,group_search_root)
    ))

  outf.write("""
    <form action="%s/groupadm/%s"
          method="POST"
          enctype="application/x-www-form-urlencoded"
          accept-charset="%s"
    >
      <input type="hidden" name="dn" value="%s">
      <p><input type="submit" value="List"> group entries below: %s.</p>
      <p>List %s groups.</p>
    </form>
  """ % (
    # form for searching group entries
    form.script_name,sid,form.accept_charset,
    w2lapp.core.utf2display(form.accept_charset,dn),
    group_search_root_field.inputHTML(title='Search root for searching group entries'),
    form.field['groupadm_view'].inputHTML(title='Group entries list',default=str(groupadm_view)),
  ))

  if groupadm_view:
    outf.write('<dl>\n')
    # Output a legend of all group entries
    for group_dn in {1:remove_groups,2:all_group_entries}[groupadm_view]:
      group_entry = all_groups_dict[group_dn]
      outf.write('<dt>%s | %s<dt>\n<dd>%s<br>\n(%s)<br>\n%s<dd>\n' % (
        ', '.join(group_entry.get('cn',[])),
        form.applAnchor('read','Read',sid,[('dn',group_dn)],title='Display group entry'),
        w2lapp.core.utf2display(form.accept_charset,group_dn),
        ', '.join(group_entry.get('objectClass',[])),
        '<br>'.join(group_entry.get('description',[]))
      ))
    outf.write('</dl>\n')

  outf.write('</div>\n')

  w2lapp.gui.PrintFooter(outf,form)
