"""
w2lapp.secinfo.py: Display (SSL) connection data

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: monitor.py,v 1.19 2008/09/01 17:39:12 michael Exp $
"""

import time,threading,utctime,ipadr,web2ldapcnf.misc,web2ldapcnf.monitor,w2lapp.core,w2lapp.gui

from pyweblib.forms import escapeHTML

from w2lapp.session import session

def w2l_Monitor(outf,command,form,env):
  """
  List several general gateway stats
  """

  if not ipadr.MatchIPAdrList(
    env['REMOTE_ADDR'],
    map(ipadr.AddrMask_Str2Tuple,web2ldapcnf.monitor.access_allowed)
  ):
    raise w2lapp.core.ErrorExit('Access denied.')

  uptime = (time.time()-w2lapp.core.startUpTime)/60

  context_menu_list = [
    form.applAnchor('','Connect',None,[]),
    form.applAnchor('locate','DNS lookup',None,[]),
  ]

  w2lapp.gui.TopSection(None,outf,form,None,None,'Monitor',[],context_menu_list)

  outf.write("""
    <div id="Message" class="Main">
      <h1>Gateway Monitor</h1>
      <dl>
        <dt>Current time:</dt>
        <dd>%s</dd>
        <dt>Startup time:</dt>
        <dd>%s</dd>
        <dt>Uptime:</dt>
        <dd>%s</dd>
        <dt>%d active threads:</dt>
        <dd>
          <ul>
            %s
          </ul>
        </dd>
        <dt>%d LDAP connections:</dt>
        <dd>
          <ul>
            %s
          </ul>
        </dd>
        <dt>Web sessions initialized:</dt>
        <dd>%d</dd>
        <dt>Web session limit:</dt>
        <dd>%d</dd>
        <dt>Time after which sessions are removed:</dt>
        <dd>%d secs</dd>
      </dl>
    </div>
    """ % (
      utctime.strftimeiso8601(time.gmtime(time.time())),
      utctime.strftimeiso8601(time.gmtime(w2lapp.core.startUpTime)),
      '%02d:%02d' % (int(uptime/60),int(uptime%60)),
      threading.activeCount(),
      '\n'.join(
        [
          '<li>%s</li>' % ''.join(
            [
              repr(t),
              ', alive'*t.isAlive(),
              ', daemon'*t.isDaemon(),
            ]
          )
          for t in threading.enumerate()
        ]
      ),
      int(len(session.sessiondict)/2),
      '\n'.join([
        '<li>%s: %s</li>' % (
          utctime.strftimeiso8601(time.gmtime(i[0])),
          escapeHTML(repr(i[1]))
        )
        for k,i in session.sessiondict.items()
        if not k.startswith('__')
      ]),
      session.sessionCounter,
      web2ldapcnf.misc.session_limit,
      session.expireRemove,
    )
  )
  w2lapp.gui.PrintFooter(outf,form)
