"""
w2lapp.referral.py: chase LDAP referrals

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: referral.py,v 1.36 2009/08/06 14:00:34 michael Exp $
"""

import ldaputil.base,w2lapp.core,w2lapp.gui,w2lapp.cnf

from ldapurl import LDAPUrl

ErrorMessageDiv = """<div id="ErrorMessage" class="Main">
<h1>Error</h1>
<p>
  %s
</p>
</div>
"""

def w2l_ChaseReferral(sid,outf,command,form,ls,dn,e):

  w2lapp.gui.TopSection(
    sid,outf,form,ls,dn,
    'Referral received',
    w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=[]
  )

  # Pull out referral LDAP URL
  try:
    ldap_url_info = [
      s.strip()
      for s in e.args[0].get('info','').split('\n')
    ]
  except ValueError:
    outf.write(
      ErrorMessageDiv % (
        'Error extracting referral LDAP URL from %s.' % (
          w2lapp.core.utf2display(form.accept_charset,repr(e))
        )
      )
    )
    w2lapp.gui.PrintFooter(outf,form)
    return
  else:
    try:
      ldap_url_info = ldap_url_info[1]
    except IndexError,e:
      outf.write(
        ErrorMessageDiv % (
          'Error extracting referral LDAP URL from %s.' % (
            w2lapp.core.utf2display(form.accept_charset,repr(ldap_url_info))
          )
        )
      )
      w2lapp.gui.PrintFooter(outf,form)
      return
    # Parse the referral LDAP URL
    try:
      referralUrl = LDAPUrl(ldap_url_info[ldap_url_info.find('ldap:'):])
    except ValueError,e:
      outf.write(ErrorMessageDiv % (
          'Error parsing referral URL %s: %s' % (
            w2lapp.core.utf2display(form.accept_charset,repr(ldap_url_info)),
            w2lapp.core.utf2display(form.accept_charset,str(e))
          )
        )
      )
      w2lapp.gui.PrintFooter(outf,form)
      return

    new_dn = referralUrl.dn

  login_template_filename = w2lapp.cnf.GetParam(ls,'login_template',None)
  login_template_filename = w2lapp.gui.GetVariantFilename(login_template_filename,form.accept_language)
  login_template_str = open(login_template_filename,'r').read()

  referral_uri = referralUrl.initializeUrl()

  login_search_root_field = w2lapp.gui.SearchRootField(form,ls,dn,name='login_search_root')
  login_fields = login_template_str % (
    form.field['login_mech'].inputHTML(),
    w2lapp.core.utf2display(form.accept_charset,ls.who),
    w2lapp.core.utf2display(form.accept_charset,w2lapp.cnf.GetParam(ls,'binddnsearch',r'(uid=%s)')),
    login_search_root_field.inputHTML(),
    'Chase Referral',
  )

  outf.write("""<div id="ErrorMessage" class="Main">
<h1>Referral received</h1>
<p>
  Referral URL: %s
</p>
"""  % (referralUrl.unparse()))
  outf.write("""
<form
  action="%s/%s/%s"
  method="POST"
  enctype="application/x-www-form-urlencoded"
  accept-charset="%s"
>
  <input type="hidden" name="host" value="%s">
  <input type="hidden" name="dn" value="%s">
  %s
"""  % (
        form.script_name,
        command,
        sid,
        form.accept_charset,
        referralUrl.hostport,
        w2lapp.core.utf2display(form.accept_charset,referralUrl.dn),
        login_fields,
      )
    )
  form.hiddenInputFields(outf,['sid','host','dn','who','cred','login_search_root'])
  outf.write('</form></div>\n')

  w2lapp.gui.PrintFooter(outf,form)
