"""
w2lapp.rename.py: modify DN of an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: rename.py,v 1.24 2009/03/30 17:25:17 michael Exp $
"""

import ldap,ldapurl,pyweblib.forms,ldaputil.base, \
       w2lapp.core,w2lapp.cnf,w2lapp.gui,w2lapp.form,w2lapp.schema.syntaxes

from w2lapp.schema.viewer import displayNameOrOIDList

def NewSuperiorField(sid,form,ls,dn,sub_schema,sup_search_url,old_superior_dn):

  class NewSuperiorSelectList(w2lapp.schema.syntaxes.DynamicDNSelectList):
    """This is a hack!!!"""
    def __init__(self,sid,form,ls,dn,schema,attrType,attrValue,lu_obj):
      self.lu_obj = lu_obj
      w2lapp.schema.syntaxes.SelectList.__init__(self,sid,form,ls,dn,schema,attrType,attrValue)
      self._doSearch(initial_attr_value_dict={'':'- Root Naming Context -'})

  if not sup_search_url is None:
    attr_inst = NewSuperiorSelectList(sid,form,ls,dn,sub_schema,'rdn',old_superior_dn,sup_search_url)
    nssf = attr_inst.formField()
    nssf.name='rename_newsuperior'
    nssf.text='New Superior DN'
  else:
    nssf = w2lapp.form.DistinguishedNameInput('rename_newsuperior','New Superior DN')
  nssf.setCharset(form.accept_charset)
  nssf.setDefault(old_superior_dn)
  return nssf # SearchSuperiors()


def w2l_Rename(sid,outf,command,form,ls,dn):

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))
  rename_supsearchurl_cfg = w2lapp.cnf.GetParam(ls,'rename_supsearchurl',{})

  if not dn:
    raise w2lapp.core.ErrorExit('Rename operation not possible at - World - or RootDSE.')

  rename_newrdn = form.getInputValue('rename_newrdn',[None])[0]
  rename_newsuperior = form.getInputValue('rename_newsuperior',[None])[0]
  rename_delold = form.getInputValue('rename_delold',['no'])[0]=='yes'
    
  if rename_newrdn:

    # ---------------------------------------
    # Rename the entry based on user's input
    # ---------------------------------------

    # Modify the RDN
    old_dn = dn
    dn = ls.renameEntry(
      dn,rename_newrdn,rename_newsuperior,delold=rename_delold
    )
    ls.setDN(dn)

    w2lapp.gui.SimpleMessage(
      sid,outf,form,ls,dn,
      'Modified RDN of entry.',
      """<p>Renamed entry.</p>
      <dl><dt>Old name:</dt><dd>%s</dd>
      <dt>New name:</dt><dd>%s</dd></dl>""" % (
        w2lapp.gui.DisplayDN(sid,form,ls,old_dn),
        w2lapp.gui.DisplayDN(sid,form,ls,dn)
      ),
      main_menu_list=w2lapp.gui.MainMenu(sid,form,ls,dn),
      context_menu_list=w2lapp.gui.ContextMenuSingleEntry(sid,form,ls,dn)
    )

  else:

    # ---------------------------------------
    # Output input form
    # ---------------------------------------

    old_rdn,old_superior = ldaputil.base.SplitRDN(dn)

    form.field['rename_newrdn'].setDefault(old_rdn)

    rename_supsearchurl = form.getInputValue('rename_supsearchurl',[None])[0]
    try:
      sup_search_url = ldapurl.LDAPUrl(rename_supsearchurl_cfg[rename_supsearchurl])
    except KeyError:
      rename_newsupfilter = form.getInputValue('rename_newsupfilter',[None])[0]
      if rename_newsupfilter!=None:
        sup_search_url = ldapurl.LDAPUrl()
        sup_search_url.urlscheme = 'ldap'
        sup_search_url.filterstr = rename_newsupfilter
        sup_search_url.dn   = form.getInputValue('rename_searchroot',[''])[0]
        sup_search_url.scope  = int(form.getInputValue('scope',[str(ldap.SCOPE_SUBTREE)])[0])
        sup_search_url.dn   = form.getInputValue('rename_searchroot',[''])[0]
      else:
        sup_search_url = None

    rename_search_root_field = w2lapp.gui.SearchRootField(form,ls,dn,name='rename_searchroot')
    rename_new_superior_field = NewSuperiorField(sid,form,ls,dn,sub_schema,sup_search_url,old_superior)

    name_forms_text = ''
    if sub_schema.sed[ldap.schema.models.NameForm]:
      # Determine if there are name forms defined for structural object class
      search_result = ls.readEntry(dn,['objectClass','structuralObjectClass','governingStructureRule'])
      if not search_result:
        # This should normally not happen, only if entry got deleted in between
        raise w2lapp.core.ErrorExit('Empty search result when reading entry to be renamed.')

      entry = ldaputil.schema.Entry(sub_schema,dn,search_result[0][1])

      # Determine possible name forms for new RDN
      rdn_options = entry.get_rdn_templates()    
      if rdn_options:
        name_forms_text = '<p>Available name forms for RDN:<br>%s</p>' % ('<br>'.join(rdn_options))

      # Determine LDAP search filter for building a select list for new superior DN
      # based on governing structure rule
      dit_structure_rule_html = ''
      dit_structure_ruleid = entry.get_governing_structure_rule(dn)
      if dit_structure_ruleid:
        sup_structural_ruleids,sup_structural_oc = sub_schema.get_superior_structural_oc_names(dit_structure_ruleid)
        if sup_structural_oc:
          superior_search_filter = '(|(%s))' % ')('.join([ '%s=*' % (oc) for oc in sup_structural_oc ])
          dit_structure_rule_html = 'DIT structure rules:<br>%s' % ('<br>'.join(displayNameOrOIDList(sid,form,dn,sub_schema,sup_structural_ruleids,ldap.schema.models.DITStructureRule)))
    # Output empty input form for new RDN
    w2lapp.gui.TopSection(
      sid,outf,form,ls,dn,
      'Rename Entry',
      w2lapp.gui.MainMenu(sid,form,ls,dn),
      context_menu_list=w2lapp.gui.ContextMenuSingleEntry(sid,form,ls,dn)
    )
    outf.write("""
<div id="Input" class="Main">

<h1>Rename/move entry</h1>

<form action="%s/rename/%s"
      method="POST"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
>
  <input type="hidden" name="dn" value="%s">
  <fieldset>
    <legend>Parameters for rename operation</legend>
    <table>
      <tr><td>%s:</td><td>%s</td></tr>
      <tr><td>%s:</td><td>%s</td></tr>
      <tr><td>%s:</td><td>%s</td></tr>
    </table>
    %s
    <input type="submit" value="Rename">
    <input type="reset" value="Reset">
  </fieldset>
</form>

<h2>Search for new superior DN</h2>

""" % (
      form.script_name,sid,form.accept_charset,
      w2lapp.core.utf2display(form.accept_charset,dn),
      form.field['rename_newrdn'].labelHTML(),
      form.field['rename_newrdn'].inputHTML(),
      rename_new_superior_field.labelHTML(),
      rename_new_superior_field.inputHTML(),
      form.field['rename_delold'].labelHTML(),
      form.field['rename_delold'].inputHTML(),
      name_forms_text,
    ))

    if rename_supsearchurl_cfg:
      rename_supsearchurl_field = pyweblib.forms.Select('rename_supsearchurl',u'LDAP URL for searching new superior entry',1,options=[])
      rename_supsearchurl_field.setOptions(rename_supsearchurl_cfg.keys())

      outf.write("""
<form action="%s/rename/%s"
      method="GET"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
>
  <input type="hidden" name="dn" value="%s">
  <fieldset>
    <legend>Pre-configured searches</legend>
    <table>
      <tr><td>%s:</td><td>%s</td></tr>
    </table>
    <input type="submit" value="Search">
    <input type="reset" value="Reset">
  </fieldset>
</form>
""" % (
        form.script_name,sid,form.accept_charset,
        w2lapp.core.utf2display(form.accept_charset,dn),
        rename_supsearchurl_field.labelHTML(labelText=u'Pre-configured search'),
        rename_supsearchurl_field.inputHTML(),
      ))

    outf.write("""
<form action="%s/rename/%s"
      method="GET"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
>
  <input type="hidden" name="dn" value="%s">
  <fieldset>
    <legend>Expert search</legend>
    <table>
      <tr><td>%s:</td><td>%s</td></tr>
      <tr><td>%s:</td><td>%s</td></tr>
      <tr><td>%s:</td><td>%s</td></tr>
    </table>
    <input type="submit" value="Search">
    <input type="reset" value="Reset">
  </fieldset>
</form>

""" % (
      form.script_name,sid,form.accept_charset,
      w2lapp.core.utf2display(form.accept_charset,dn),
      form.field['rename_newsupfilter'].labelHTML(labelText=u'Search filter'),
      form.field['rename_newsupfilter'].inputHTML(),
      rename_search_root_field.labelHTML(labelText=u'Search root'),
      rename_search_root_field.inputHTML(),
      form.field['scope'].labelHTML(),
      form.field['scope'].inputHTML(),
    ))

    outf.write('</div>')
    w2lapp.gui.PrintFooter(outf,form)


