"""
web2ldap plugin classes for Novell eDirectory
(see draft-sermersheim-nds-ldap-schema)

$Id: edirectory.py,v 1.17 2009/08/07 13:59:52 michael Exp $
"""

from mspki.util import HexString

from pyweblib.forms import escapeHTML

from w2lapp.schema.syntaxes import DirectoryString,Integer,OctetString, \
  PostalAddress,PrintableString,Certificate,CertificateRevocationList, \
  MultilineText,OID,DynamicDNSelectList,NullTerminatedDirectoryString, \
  Binary,syntax_registry

from w2lapp.gui import DisplayDN

class CaseIgnoreList(PostalAddress):
  oid = '2.16.840.1.113719.1.1.5.1.6'
  desc = 'Case Ignore List'


class TaggedData(OctetString):
  oid = '2.16.840.1.113719.1.1.5.1.12'
  desc = 'Tagged Data'


class OctetList(OctetString):
  oid = '2.16.840.1.113719.1.1.5.1.13'
  desc = 'Octet List'


class TaggedString(DirectoryString):
  oid = '2.16.840.1.113719.1.1.5.1.14'
  desc = 'Tagged String'


class DollarSeparatedMultipleLines(PostalAddress):
  oid = '2.16.840.1.113719.1.1.5.1.6'
  desc = '$-separated string'


class IndexDefinition(DollarSeparatedMultipleLines):
  """
  Version: 0 (reserved for future use)
  Name: description of index
  State: 0-suspend, 1-bringing, 2-online, 3-pending
  Matching Rule: 0-value, 1-presence, 2-substring
  Type: 0-user defined
  Value State: 1-added from server
  NDS Attribute Name
  """
  oid = 'IndexDefinition-oid'
  desc = 'Index Definition'

  def displayValue(self,valueindex=0,commandbutton=0):
    try:
      version,index_name,state,matching_rule,index_type,value_state,nds_attribute_name = self.attrValue.split('$')
      version = int(version)
      index_name = unicode(index_name,self._ls.charset)
      state = int(state)
      matching_rule = int(matching_rule)
      index_type = int(index_type)
      value_state = int(value_state)
      nds_attribute_name = unicode(nds_attribute_name,self._ls.charset)
    except ValueError,UnicodeDecodeError:
      return DollarSeparatedMultipleLines.displayValue(self,valueindex,commandbutton)
    else:
      return """
        <table noborder>
          <tr><td>Version:</td><td>%s</td></tr>
          <tr><td>Name:</td><td>%s</td></tr>
          <tr><td>State:</td><td>%s</td></tr>
          <tr><td>Matching Rule:</td><td>%s</td></tr>
          <tr><td>Type:</td><td>%s</td></tr>
          <tr><td>Value State:</td><td>%s</td></tr>
          <tr><td>NDS Attribute Name</td><td>%s</td></tr>
        </table>""" % (
          version,
          index_name.encode(self._form.accept_charset),
          {0:'suspend',1:'bringing',2:'online',3:'pending'}.get(state,str(state)),
          {0:'value',1:'presence',2:'substring'}.get(matching_rule,str(matching_rule)),
          {0:'user defined'}.get(index_type,str(index_type)),
          {1:'added from server'}.get(value_state,str(value_state)),
          nds_attribute_name.encode(self._form.accept_charset),
        )

syntax_registry.registerAttrType(
  IndexDefinition.oid,[
    '2.16.840.1.113719.1.1.4.1.512', # indexDefinition
  ]
)


class TaggedNameAndString(DirectoryString):
  oid = '2.16.840.1.113719.1.1.5.1.15'
  desc = 'Tagged Name And String'

  def displayValue(self,valueindex=0,commandbutton=0):
    try:
      ind2 = self.attrValue.rindex('#')
      ind1 = self.attrValue.rindex('#',0,ind2-1)
    except ValueError:
      return DirectoryString.displayValue(self,sid,form,ls,dn,valueindex,commandbutton)
    dn = unicode(self.attrValue[0:ind1],self._ls.charset)
    number = self.attrValue[ind1+1:ind2]
    dstring = self.attrValue[ind2+1:]
    return '<dl><dt>name:</dt><dd>%s</dd><dt>number:</dt><dd>%s</dd><dt>dstring:</dt><dd>%s</dd></dl>' % (
      DisplayDN(self._sid,self._form,self._ls,dn,commandbutton=commandbutton),
      number,
      '<code><font size="-1">%s</font></code>' % HexString(
        dstring,delimiter=':',wrap=64,linesep='<br>'
      ),
    )

class NDSReplicaPointer(OctetString):
  oid = '2.16.840.1.113719.1.1.5.1.16'
  desc = 'NDS Replica Pointer'


class NDSACL(DirectoryString):
  oid = '2.16.840.1.113719.1.1.5.1.17'
  desc = 'NDS ACL'


class NDSTimestamp(PrintableString):
  oid = '2.16.840.1.113719.1.1.5.1.19'
  desc = 'NDS Timestamp'


class Counter(Integer):
  oid = '2.16.840.1.113719.1.1.5.1.22'
  desc = 'Counter (NDS)'


class TaggedName(DirectoryString):
  oid = '2.16.840.1.113719.1.1.5.1.23'
  desc = 'Tagged Name'


class TypedName(DirectoryString):
  oid = '2.16.840.1.113719.1.1.5.1.25'
  desc = 'Typed Name'


try:
  from w2lapp.schema.syntaxes import XmlValue
except ImportError:
  pass
else:
  syntax_registry.registerAttrType(
    XmlValue.oid,[
      '2.16.840.1.113719.1.1.4.1.295', # emboxConfig
      '2.16.840.1.113719.1.14.4.1.3',  # XmlData
      '2.16.840.1.113719.1.14.4.1.8',  # DirXML-ShimConfigInfo
      '2.16.840.1.113719.1.14.4.1.11', # DirXML-DriverFilter
      '2.16.840.1.113719.1.14.4.1.24', # DirXML-DriverCacheLimit
      '2.16.840.1.113719.1.14.4.1.29', # DirXML-ApplicationSchema
      '2.16.840.1.113719.1.14.4.1.54', # DirXML-ConfigValues
      '2.16.840.1.113719.1.14.4.1.56', # DirXML-ConfigManifest
      '2.16.840.1.113719.1.14.4.1.58', # DirXML-EngineControlValues
      '2.16.840.1.113719.1.14.4.1.82', # DirXML-PersistentData
    ]
  )


class SnmpTrapDescription(MultilineText):
  oid = 'SnmpTrapDescription'
  desc = 'SNMP Trap Description'
  lineSep = '\x00'
  cols = 30

syntax_registry.registerAttrType(
  SnmpTrapDescription.oid,[
    '2.16.840.1.113719.1.6.4.4', # snmpTrapDescription
  ]
)


class NspmPasswordPolicyDN(DynamicDNSelectList):
  oid = 'NspmPasswordPolicyDN-oid'
  desc = 'DN of the nspmPasswordPolicy entry'
  ldap_url = 'ldap:///cn=Password Policies,cn=Security?cn?sub?(objectClass=nspmPasswordPolicy)'

syntax_registry.registerAttrType(
  NspmPasswordPolicyDN.oid,[
    '2.16.840.1.113719.1.39.43.4.6', # nspmPasswordPolicyDN
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))


# Workarounds for eDirectory

syntax_registry.registerAttrType(
  Certificate.oid,[
    '2.16.840.1.113719.1.48.4.1.3', # nDSPKIPublicKeyCertificate
  ]
)

syntax_registry.registerAttrType(
  CertificateRevocationList.oid,[
    '2.16.840.1.113719.1.48.4.1.34', # certificateRevocationList in Novell eDirectory
  ]
)

syntax_registry.registerAttrType(
  OID.oid,
  [
    'supportedGroupingTypes',
  ]
)

syntax_registry.registerAttrType(
  NullTerminatedDirectoryString.oid,[
    '2.16.840.1.113719.1.27.4.42', # extensionInfo
  ]
)

syntax_registry.registerAttrType(
  Binary.oid,[
    '2.16.840.1.113719.1.48.4.1.4', # nDSPKICertificateChain
    '2.16.840.1.113719.1.48.4.1.2', # nDSPKIPrivateKey
    '2.16.840.1.113719.1.48.4.1.1', # nDSPKIPublicKey
  ]
)
