"""
web2ldap plugin classes for attributes defined in draft-behera-ldap-password-policy

$Id: opends.py,v 1.1 2009/07/17 19:21:48 michael Exp $
"""

import re

from w2lapp.schema.syntaxes import LDAPSyntax,DirectoryString,SelectList,DynamicDNSelectList,syntax_registry


class OpenDSCfgPasswordPolicy(DynamicDNSelectList):
  oid = 'OpenDSCfgPasswordPolicy-oid'
  desc = 'DN of the ds-cfg-password-policy entry'
  ldap_url = 'ldap:///cn=Password Policies,cn=config?cn?one?(objectClass=ds-cfg-password-policy)'

syntax_registry.registerAttrType(
  OpenDSCfgPasswordPolicy.oid,[
    '1.3.6.1.4.1.26027.1.1.161', # ds-cfg-default-password-policy
  ]
)


class OpenDSCfgPasswordStorageScheme(DynamicDNSelectList):
  oid = 'OpenDSCfgPasswordStorageScheme-oid'
  desc = 'DN of the ds-cfg-password-storage-scheme entry'
  ldap_url = 'ldap:///cn=Password Storage Schemes,cn=config?cn?one?(objectClass=ds-cfg-password-storage-scheme)'

syntax_registry.registerAttrType(
  OpenDSCfgPasswordStorageScheme.oid,[
    '1.3.6.1.4.1.26027.1.1.137', # ds-cfg-default-password-storage-scheme
  ]
)

class OpenDSCfgPasswordGenerator(DynamicDNSelectList):
  oid = 'OpenDSCfgPasswordGenerator-oid'
  desc = 'DN of the ds-cfg-password-generator entry'
  ldap_url = 'ldap:///cn=Password Generators,cn=config?cn?one?(objectClass=ds-cfg-password-generator)'

syntax_registry.registerAttrType(
  OpenDSCfgPasswordGenerator.oid,[
    '1.3.6.1.4.1.26027.1.1.153', # ds-cfg-password-generator
  ]
)


class OpenDSCfgIdentityMapper(DynamicDNSelectList):
  oid = 'OpenDSCfgIdentityMapper-oid'
  desc = 'DN of the ds-cfg-identity-mapper entry'
  ldap_url = 'ldap:///cn=Identity Mappers,cn=config?cn?one?(objectClass=ds-cfg-identity-mapper)'

syntax_registry.registerAttrType(
  OpenDSCfgIdentityMapper.oid,[
    '1.3.6.1.4.1.26027.1.1.114', # ds-cfg-proxied-authorization-identity-mapper
  ]
)


class OpenDSCfgInvalidSchemaBehaviour(SelectList):
  oid = 'OpenDSCfgInvalidSchemaBehaviour-oid'
  desc = 'Specifies how OpenDS behaves in case of schema errors'
  attr_value_dict = {
    'reject':'reject',
    'default':'default',
    'accept':'accept',
    'warn':'warn',
  }

syntax_registry.registerAttrType(
  OpenDSCfgInvalidSchemaBehaviour.oid,[
    '1.3.6.1.4.1.26027.1.1.31', # ds-cfg-invalid-attribute-syntax-behavior
    '1.3.6.1.4.1.26027.1.1.88', # ds-cfg-single-structural-objectclass-behavior
  ]
)


class OpenDSCfgWritabilityMode(SelectList):
  oid = 'OpenDSCfgWritabilityMode-oid'
  desc = 'Specifies the kinds of write operations the Directory Server can process.'
  attr_value_dict = {
    'disabled':'all write operations are rejected',
    'enabled':'all write operations are processed',
    'internal-only':'write operations requested as internal/sync operations are processed',
  }

syntax_registry.registerAttrType(
  OpenDSCfgWritabilityMode.oid,[
    '1.3.6.1.4.1.26027.1.1.123', # ds-cfg-writability-mode  
  ]
)


class OpenDSCfgTimeInterval(DirectoryString):
  oid = 'OpenDSCfgTimeInterval-oid'
  desc = 'A time interval consisting of integer value and time unit'
  reObj=re.compile('^[0-9]+ (seconds|minutes|hours|days)$')

syntax_registry.registerAttrType(
  OpenDSCfgTimeInterval.oid,[
    '1.3.6.1.4.1.26027.1.1.142', # ds-cfg-idle-lockout-interval
    '1.3.6.1.4.1.26027.1.1.145', # ds-cfg-lockout-duration
    '1.3.6.1.4.1.26027.1.1.147', # ds-cfg-lockout-failure-expiration-interval
    '1.3.6.1.4.1.26027.1.1.148', # ds-cfg-max-password-age
    '1.3.6.1.4.1.26027.1.1.149', # ds-cfg-max-password-reset-age
    '1.3.6.1.4.1.26027.1.1.150', # ds-cfg-min-password-age
    '1.3.6.1.4.1.26027.1.1.152', # ds-cfg-password-expiration-warning-interval
    '1.3.6.1.4.1.26027.1.1.375', # ds-cfg-password-history-duration
    '1.3.6.1.4.1.26027.1.1.115', # ds-cfg-time-limit
  ]
)
  

# Register all syntax classes in this module
for name in dir():
  syntax_registry.registerSyntaxClass(eval(name))

