"""
web2ldap plugin classes for OpenLDAP

$Id: openldap.py,v 1.5 2009/07/16 15:00:07 michael Exp $
"""

from w2lapp.schema.syntaxes import DistinguishedName,DirectoryString,Uri,SelectList,syntax_registry

import ldap.filter
import w2lapp.gui

#---------------------------------------------------------------------------
# slapo-syncprov
#---------------------------------------------------------------------------


class EntryCSN(DirectoryString):
  oid = '1.3.6.1.4.1.4203.666.1.7'
  desc = 'OpenLDAP EntryCSN'


#---------------------------------------------------------------------------
# back-config
#---------------------------------------------------------------------------


class OlcSubordinate(SelectList):
  oid = 'OlcSubordinate-oid'
  desc = 'Indicates whether backend is subordinate'
  attr_value_dict = {
    '':'FALSE (not set)',
    'TRUE':'TRUE',
    'advertise':'advertise',
  }

syntax_registry.registerAttrType(
  OlcSubordinate.oid,[
    'olcSubordinate', # olcSubordinate
  ]
)


#---------------------------------------------------------------------------
# slapo-accesslog
#---------------------------------------------------------------------------


class AuditContext(DistinguishedName):
  oid = 'AuditContext'
  desc = 'OpenLDAP DN pointing to audit naming context'

  def displayValue(self,valueindex=0,commandbutton=0):
    r = [DistinguishedName.displayValue(self,valueindex,commandbutton)]
    if commandbutton:
      r.extend([
        self._form.applAnchor(
          'search','Audit all DB',self._sid,
          [
            ('dn',self.attrValue),
            ('search_filterstr','(objectClass=auditObject)'),
            ('scope',str(ldap.SCOPE_ONELEVEL)),
          ],
          title='Complete audit trail of current database',
        ),
        self._form.applAnchor(
          'search','Audit writes to DB',self._sid,
          [
            ('dn',self.attrValue),
            ('search_filterstr','(objectClass=auditWriteObject)'),
            ('scope',str(ldap.SCOPE_ONELEVEL)),
          ],
          title='Audit trail of write access to current database',
        ),
      ])
    return ' | '.join(r)

syntax_registry.registerAttrType(
  AuditContext.oid,['1.3.6.1.4.1.4203.666.11.5.1.30','auditContext'] # auditContext
)


class ReqResult(SelectList):
  oid = 'ReqResult-oid'
  desc = 'LDAPv3 declaration of resultCode in (see RFC 4511)'
  attr_value_dict = {
    '':'- not set -',
    '0':'success',
    '1':'operationsError',
    '2':'protocolError',
    '3':'timeLimitExceeded',
    '4':'sizeLimitExceeded',
    '5':'compareFalse',
    '6':'compareTrue',
    '7':'authMethodNotSupported',
    '8':'strongerAuthRequired',
    '9':'reserved',
    '10':'referral',
    '11':'adminLimitExceeded',
    '12':'unavailableCriticalExtension',
    '13':'confidentialityRequired',
    '14':'saslBindInProgress',
    '16':'noSuchAttribute',
    '17':'undefinedAttributeType',
    '18':'inappropriateMatching',
    '19':'constraintViolation',
    '20':'attributeOrValueExists',
    '21':'invalidAttributeSyntax',
    '32':'noSuchObject',
    '33':'aliasProblem',
    '34':'invalidDNSyntax',
    '35':'reserved for undefined isLeaf',
    '36':'aliasDereferencingProblem',
    '48':'inappropriateAuthentication',
    '49':'invalidCredentials',
    '50':'insufficientAccessRights',
    '51':'busy',
    '52':'unavailable',
    '53':'unwillingToPerform',
    '54':'loopDetect',
    '64':'namingViolation',
    '65':'objectClassViolation',
    '66':'notAllowedOnNonLeaf',
    '67':'notAllowedOnRDN',
    '68':'entryAlreadyExists',
    '69':'objectClassModsProhibited',
    '70':'reserved for CLDAP',
    '71':'affectsMultipleDSAs',
    '80':'other',
  }

syntax_registry.registerAttrType(
  ReqResult.oid,[
    'reqResult', # reqResult
  ]
)


#---------------------------------------------------------------------------
# General
#---------------------------------------------------------------------------


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))

syntax_registry.registerAttrType(
  DistinguishedName.oid,[
    'entryDN','monitorConnectionAuthzDN',
    'configContext','monitorContext','reqDN','reqAuthzID'
  ]
)

syntax_registry.registerAttrType(
  Uri.oid,['monitorConnectionListener']
)

