"""
web2ldap plugin classes for attributes defined in SCHAC

See http://www.terena.org/activities/tf-emc2/schac.html

$Id: schac.py,v 1.7 2009/06/14 11:25:07 michael Exp $
"""

import re
from w2lapp.schema.syntaxes import DirectoryString,NumericString,CountryString,DNSDomain,syntax_registry
from w2lapp.schema.plugins.msperson import Gender


syntax_registry.registerAttrType(
  CountryString.oid,[
    '1.3.6.1.4.1.25178.1.2.5',  # schacCountryOfCitizenship 
    '1.3.6.1.4.1.25178.1.2.11', # schacCountryOfResidence
  ]
)

syntax_registry.registerAttrType(
  DNSDomain.oid,[
    '1.3.6.1.4.1.25178.1.2.9', # schacHomeOrganization
  ]
)

class SchacMotherTongue(NumericString):
  oid = 'SchacMotherTongue-oid'
  desc = 'Language tag of the language a person learns first (see RFC 3066).'
  reObj = re.compile('^[a-zA-Z]{2,8}(-[a-zA-Z0-9]{2,8})*$')

syntax_registry.registerAttrType(
  SchacMotherTongue.oid,[
    '1.3.6.1.4.1.25178.1.2.1', # schacMotherTongue
  ]
)


syntax_registry.registerAttrType(
  Gender.oid,[
    '1.3.6.1.4.1.25178.1.2.2', # schacGender
  ]
)


class SchacDateOfBirth(NumericString):
  oid = 'SchacDateOfBirth-oid'
  desc = 'Date of birth: syntax YYYYMMDD'
  maxLen = 8
  reObj = re.compile('^[0-9]{4}[0-1][0-9][0-3][0-9]$')

syntax_registry.registerAttrType(
  SchacDateOfBirth.oid,[
    '1.3.6.1.4.1.25178.1.2.3', # schacDateOfBirth
  ]
)


class SchacYearOfBirth(NumericString):
  oid = 'SchacYearOfBirth-oid'
  desc = 'Year of birth: syntax YYYY'
  maxLen = 4
  reObj = re.compile('^[0-9]{4}$')

syntax_registry.registerAttrType(
  SchacYearOfBirth.oid,[
    '1.3.6.1.4.1.25178.1.0.3', # schacYearOfBirth
  ]
)


class SchacUrn(DirectoryString):
  oid = 'SchacUrn-oid'
  desc = 'Generic URN for SCHAC'
  reObj = re.compile('^urn:mace:terena.org:schac:.+$')

syntax_registry.registerAttrType(
  SchacUrn.oid,[
    '1.3.6.1.4.1.25178.1.2.10', # schacHomeOrganizationType
    '1.3.6.1.4.1.25178.1.2.13', # schacPersonalPosition
    '1.3.6.1.4.1.25178.1.2.14', # schacPersonalUniqueCode
    '1.3.6.1.4.1.25178.1.2.15', # schacPersonalUniqueID
    '1.3.6.1.4.1.25178.1.2.19', # schacUserStatus
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))
