"""
This script checks all the prerequisites for running
a particular release version of web2ldap.

$Id: checkinst.py,v 1.17 2008/07/13 16:58:38 michael Exp $
"""

import sys,os,imp

print '*** Checking installation prerequisites for web2ldap ***'

exec_startdir = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
sys.path.insert(0,os.path.join(exec_startdir,'pylib'))

print """Make sure to run this script with the same Python
interpreter executable you plan to invoke web2ldap with!

***Python interpreter***"""
print 'sys.executable',repr(sys.executable)
print 'sys.prefix',repr(sys.prefix)
print 'sys.exec_prefix',repr(sys.exec_prefix)
print 'sys.version',repr(sys.version)
if sys.version<'2.3':
  print 'Python 2.3.x or newer is required! Detected %s.' % (repr(sys.version))

print 'sys.platform',repr(sys.platform)
print 'os.name',repr(os.name)

print '*** Installed modules ***'

modules = {}
missing = []

for modulename,mandantory,url,desc in [
  ('w2lapp',1,'http://www.web2ldap.de','Internal web2ldap application module'),
  ('ldap',1,'http://python-ldap.sf.net','Module package for accessing LDAP servers'),
  ('ldap.sasl',1,'http://python-ldap.sf.net','Sub-module for SASL support with LDAP'),
  ('ldap.schema',1,'http://python-ldap.sf.net','Sub-module for LDAP sub schema support'),
  ('ldap.async',1,'http://python-ldap.sf.net','Sub-module for LDAP async searches'),
  ('ldapurl',1,'http://python-ldap.sf.net','LDAP URL support'),
  ('ldif',1,'http://python-ldap.sf.net','LDIF support'),
  ('xml.sax',1,'http://www.python.org',"Python's built-in XML support"),
  ('xml.sax.handler',1,'http://www.python.org',"Python's built-in XML support"),
  ('dsml',1,'http://python-ldap.sf.net','DSMLv1 support'),
  ('pyweblib',1,'http://www.stroeder.com/pylib/PyWebLib/','Yet another web application module package'),
  ('fcgi',0,'http://alldunn.com/python/fcgi.py','FCGI module (e.g., Apache with mod_fastcgi)'),
  ('md4',0,'http://www.geocities.com/rozmanov/python/md4.html','MD4 hash module for synced Samba passwords'),
  ('Crypto.Hash',0,'http://www.amk.ca/python/code/crypto.html','PyCrypto hash module for synced Samba passwords'),
  ('Crypto.Cipher',0,'http://www.amk.ca/python/code/crypto.html','PyCrypto cipher module for synced Samba passwords'),
  ('crypt',0,'http://www.python.org','Unix crypt hash module for client-side hashed LDAP passwords'),
  ('hashlib',0,'http://www.python.org','Module useful for generating hashed LDAP passwords'),
  ('md5',0,'http://www.python.org','MD5 hash module for client-side hashed LDAP passwords'),
  ('sha',0,'http://www.python.org','SHA-1 hash module for client-side hashed LDAP passwords'),
  ('DNS',0,'http://pydns.sf.net','module for DNS lookups, e.g. SRV RRs'),
  ('PIL',0,'http://www.pythonware.com/products/pil/index.htm','Python Imaging Library'),
  ('psyco',0,'http://psyco.sourceforge.net/','Performance optimization module (special pre-compiler)'),
]:
  error_text_prefix={0:'Warning!',1:'Fatal Error!'}[mandantory]
  m_list = modulename.split('.')
  i = 0
  while i<len(m_list):
    m = m_list[i]
    f = None
    try:
      f, pathname, description = imp.find_module(m,sys.path)
    except ImportError,e:
      missing.append(m)
      print '%s Module %s (%s) not found: %s' % (error_text_prefix,modulename,desc,str(e))
    else:
      if i+1<len(m_list):
        sys.path.append(pathname)
        i += 1
        continue
      try:
        modules[m] = imp.load_module(m, f, pathname, description)
      except ImportError,e:
        missing.append(m)
        print '%s Unable to load module %s (%s): %s' % (error_text_prefix,modulename,desc,str(e))
      else:
        try:
          version = modules[m].__version__
        except AttributeError:
          version = 'unspecified'
        print 'Module %s (version %s) successfully imported: %s' % (modulename,version,desc)
    if f:
      f.close()
    i += 1

try:
  import ldap,_ldap
except ImportError:
  pass
else:
  # Display version numbers of OpenLDAP libs
  print '*** OpenLDAP libs ***'
  print 'ldap.API_VERSION: %s' % (ldap.API_VERSION)
  print 'ldap.VENDOR_VERSION: %s' % (ldap.VENDOR_VERSION)
  # Check whether built with SSL/TLS (OpenSSL)
  print '*** Suport for SSL/TLS ***'
  if hasattr(ldap,'TLS_AVAIL'):
    print 'ldap.TLS_AVAIL: %s' % (ldap.TLS_AVAIL)
  # Check whether built with SASL (Cyrus-SASL)
  print '*** Suport for SASL ***'
  if hasattr(ldap,'SASL_AVAIL'):
    print 'ldap.SASL_AVAIL: %s' % (ldap.TLS_AVAIL)
